# Generated by roxygen2: do not edit by hand

S3method(print,NNS.data.frame)
S3method(print,NNS.matrix)
export(Co.LPM)
export(Co.LPM_nD)
export(Co.UPM)
export(Co.UPM_nD)
export(D.LPM)
export(D.UPM)
export(DPM_nD)
export(LPM)
export(LPM.VaR)
export(LPM.ratio)
export(NNS.ANOVA)
export(NNS.ARMA)
export(NNS.ARMA.optim)
export(NNS.CDF)
export(NNS.FSD)
export(NNS.FSD.uni)
export(NNS.MC)
export(NNS.SD.cluster)
export(NNS.SD.efficient.set)
export(NNS.SS)
export(NNS.SSD)
export(NNS.SSD.uni)
export(NNS.TSD)
export(NNS.TSD.uni)
export(NNS.VAR)
export(NNS.boost)
export(NNS.caus)
export(NNS.copula)
export(NNS.dep)
export(NNS.diff)
export(NNS.distance)
export(NNS.gravity)
export(NNS.meboot)
export(NNS.mode)
export(NNS.moments)
export(NNS.norm)
export(NNS.part)
export(NNS.reg)
export(NNS.rescale)
export(NNS.seas)
export(NNS.stack)
export(PM.matrix)
export(UPM)
export(UPM.VaR)
export(UPM.ratio)
export(dy.d_)
export(dy.dx)
import(Rcpp, except = LdFlags)
import(RcppParallel)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,boxplot)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,matplot)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,segments)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(stats,.preformat.ts)
importFrom(stats,acf)
importFrom(stats,aggregate)
importFrom(stats,approx)
importFrom(stats,as.dist)
importFrom(stats,ave)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,density)
importFrom(stats,dexp)
importFrom(stats,dlnorm)
importFrom(stats,dnorm)
importFrom(stats,dt)
importFrom(stats,ecdf)
importFrom(stats,embed)
importFrom(stats,fivenum)
importFrom(stats,frequency)
importFrom(stats,hat)
importFrom(stats,hclust)
importFrom(stats,is.ts)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,optim)
importFrom(stats,optimize)
importFrom(stats,poly)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,resid)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,smooth.spline)
importFrom(stats,start)
importFrom(stats,t.test)
importFrom(stats,time)
importFrom(stats,ts)
importFrom(stats,uniroot)
importFrom(stats,var)
importFrom(stats,wilcox.test)
importFrom(utils,combn)
importFrom(utils,flush.console)
importFrom(utils,globalVariables)
importFrom(utils,head)
importFrom(utils,tail)
useDynLib(NNS)
useDynLib(NNS, .registration = TRUE)
