#' @name NNS
#' 
#' @title NNS: Nonlinear Nonparametric Statistics
#'
#' @description Nonlinear nonparametric statistics using partial moments.  Partial moments are the elements of variance and asymptotically approximate the area of f(x).  These robust statistics provide the basis for nonlinear analysis while retaining linear equivalences.  NNS offers: Numerical integration, Numerical differentiation, Clustering, Correlation, Dependence, Causal analysis, ANOVA, Regression, Classification, Seasonality, Autoregressive modeling, Normalization and Stochastic dominance.  All routines based on: Viole, F. and Nawrocki, D. (2013), Nonlinear Nonparametric Statistics: Using Partial Moments (ISBN: 1490523995, 2nd edition: \url{https://ovvo-financial.github.io/NNS/book/}).
#'
#' @docType package
#' @useDynLib NNS
#' @keywords internal
#' @aliases NNS-package
#'
"_PACKAGE"