% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NNS_Distance.R
\name{NNS.distance}
\alias{NNS.distance}
\title{NNS Distance}
\usage{
NNS.distance(rpm, dist.estimate, k = "all", class = NULL)
}
\arguments{
\item{rpm}{REGRESSION.POINT.MATRIX from \link{NNS.reg}}

\item{dist.estimate}{Vector to generate distances from.}

\item{k}{\code{n.best} from \link{NNS.reg}}

\item{class}{if classification problem.}
}
\value{
Returns sum of weighted distances.
}
\description{
Internal kernel function for NNS multivariate regression \link{NNS.reg} parallel instances.
}
